; ===================================================================
; Project: Settings Library
; Description: Functions for retrieving settings.
; Author: Jason Boggs <vampire DOT valik AT gmail DOT com>
; ===================================================================
#include-once

#Region Members Exported
#cs Exported Functions
_SettingBuildDir() - Changes the working directory to the root build directory.
_SettingGet($vSetting, $vDefault = "", $bNumber = False, $sSection = $g_sSection) - Retrieves a setting.
_SettingGetSection($sSection) - Returns all key, value pairs in a section.
_SettingSet($vSetting, $vValue, $sSection = $g_sSection) - Sets a setting.
_SettingSetEnv($vSetting, $vValue = "") - Sets a setting in the environment which overrides the INI file.
_SettingGetIniFile() - Returns the path to the configuration file.
#ce Exported Functions
#EndRegion Members Exported

#Region Includes
#EndRegion Includes

#Region Global Variables
Global Const $g_sIni = @ScriptDir & "\config.ini" ; DO NOT USE DIRECTLY (See _SettingGetIniFile())
Global Const $g_sSection = "Build"

; Settings Constants
Global Const $SETTING_BUILDDIR = "BuildDir"
Global Const $SETTING_COMPILER = "Compiler"
Global Const $SETTING_SIGN = "Sign"
Global Const $SETTING_RUNTEST = "RunTest"
Global Const $SETTING_ADMINTEST = "AdminTest"
Global Const $SETTING_RUNPERFORMANCE = "RunPerf"
Global Const $SETTING_STOPTEST = "StopTestFirstError"
Global Const $SETTING_ARCHIVE = "Archive"
Global Const $SETTING_PLATFORM = "Platform"
Global Const $SETTING_CONFIGURATION = "Configuration"
Global Const $SETTING_BUILDX64 = "Buildx64"
Global Const $SETTING_REBUILD = "RebuildAlways"
Global Const $SETTING_AUTOITX3TEST = "AutoItX3Test"
Global Const $SETTING_YEAR = "Year"
Global Const $SETTING_SVNCLIENT = "SubversionClient"
Global Const $SETTING_BUILDTYPE = "BuildType"
Global Const $SETTING_PROFILE = "Profile"
Global Const $SETTING_SCPHOSTKEY = "SCPHostKey"
Global Const $SETTING_SCPPRIVATEKEY = "SCPPrivateKey"
Global Const $SETTING_SCPSYNCBYSIZE = "SCPSyncBySize"
Global Const $SETTING_SCPSYNCBYDATE = "SCPSyncByDate"
Global Const $SETTING_PFXPASSWORD = "PFXPassword"
Global Const $SETTING_REBUILDHELPFILES = "RebuildHelpFiles"

; Define the Build Types supported.
Global Enum $BUILDTYPE_RELEASE, $BUILDTYPE_BETA, $BUILDTYPE_RELEASECANDIDATE
#EndRegion Global Variables

#Region Public Members

#Region _SettingBuildDir()
; ===================================================================
; _SettingBuildDir()
;
; Changes the working directory to the root build directory.
; Parameters:
;	None.
; Returns:
;	None.
; ===================================================================
Func _SettingBuildDir()
	FileChangeDir(@ScriptDir)
	FileChangeDir(_SettingGet($SETTING_BUILDDIR))
	Return @WorkingDir
EndFunc   ;==>_SettingBuildDir
#EndRegion _SettingBuildDir()

#Region _SettingGet()
; ===================================================================
; _SettingGet($vSetting, $vDefault = "", $bNumber = False, $sSection = $g_sSection)
;
; Retrieves a setting.
; Parameters:
;	$vSetting - IN - The name of the setting to obtain.
;	$vDefault - IN/OPTIONAL - The default value to return if the setting can't be found.
;	$bNumber - IN/OPTIONAL - If true, cast the result to a number.
;	$sSection - IN/OPTIONAL - Allows the section to be overriden.
; Returns:
;	Success: The setting optionally cast to a number.
;	Failure: The default value.
; ===================================================================
Func _SettingGet($vSetting, $vDefault = "", $bNumber = False, $sSection = Default, $bRedirected = False)
	; Check the environment first, it overrides.
	Local $vResult = EnvGet("Env" & $vSetting)
	; Nothing in the environment, read the INI file.
	If $sSection = Default Then $sSection = $g_sSection
	If Not $vResult Then $vResult = IniRead(_SettingGetIniFile($bRedirected), $sSection, $vSetting, $vDefault)
	If $bNumber Then Return Number($vResult)
	Return $vResult
EndFunc   ;==>_SettingGet
#EndRegion _SettingGet()

#Region _SettingGetSection()
; ===================================================================
; _SettingGetSection($sSection)
;
; Returns all key, value pairs in a section.
; Parameters:
;	$sSection - IN - The section to return the key, value pairs for.
; Returns:
;	Success - An array of key, value pairs.
;	Failure - An empty string and @error set to non-zero.
; ===================================================================
Func _SettingGetSection($sSection)
	Local $aRet = IniReadSection(_SettingGetIniFile(), $sSection)
	Return SetError(@error, @extended, $aRet)
EndFunc   ;==>_SettingGetSection
#EndRegion _SettingGetSection()

#Region _SettingSet()
; ===================================================================
; _SettingSet($vSetting, $vValue, $sSection = $g_sSection)
;
; Sets a setting.
; Parameters:
;	$vSetting - IN - The name of the setting to set.
;	$vValue - IN - The data to set.
;	$sSection - IN/OPTIONAL - Allows the section to be overriden.
; Returns:
;	None.
; ===================================================================
Func _SettingSet($vSetting, $vValue, $sSection = $g_sSection)
	IniWrite(_SettingGetIniFile(), $sSection, $vSetting, $vValue)
EndFunc   ;==>_SettingSet
#EndRegion _SettingSet()

#Region _SettingSetEnv()
; ===================================================================
; _SettingSetEnv($vSetting, $vValue = "")
;
; Sets a setting in the environment which overrides the INI file.
; Parameters:
;	$vSetting - IN - The name of the setting to set.
;	$vValue - IN/OPTIONAL - IN - The data to set.  If not specified then the environment override
;		will be deleted.
; Returns:
;	None.
; ===================================================================
Func _SettingSetEnv($vSetting, $vValue = "")
	$vSetting = "Env" & $vSetting
	If @NumParams = 1 Then
		EnvSet($vSetting)
	Else
		EnvSet($vSetting, $vValue)
	EndIf
EndFunc   ;==>_SettingSetEnv
#EndRegion _SettingSetEnv()

#Region _SettingGetIniFile()
; ===================================================================
; _SettingGetIniFile()
;
; Returns the path to the configuration file.
; Parameters:
;	None.
; Returns:
;	The path to the configuration file.
; ===================================================================
Func _SettingGetIniFile($bRedirected = False)
	If $bRedirected Then
		Local $sTemp = IniRead($g_sIni, "IniVersion", "RedirectedIni", "")
		If $sTemp Then
			; Config.ini need to be redirected
			If Not FileExists($sTemp) Then
				; use Config.ini located in the trunk\_build
				$sTemp = IniRead($g_sIni, "Build", "BuildDir", "") & "\" & $sTemp
			EndIf
			Return $sTemp
		EndIf
	EndIf

	Return $g_sIni
EndFunc   ;==>_SettingGetIniFile
#EndRegion _SettingGetIniFile()

#EndRegion Public Members
