; ===================================================================
; Project: Changelog
; Description: Creates a valid HTML5 changelog file.
; Author: guinness
; ===================================================================

#include '..\..\_build\include\MiscLib.au3'
#include '..\..\_build\include\OutputLib.au3'

FileChangeDir(@ScriptDir) ; As the build scripts have a habit of changing the working directory.

Opt('TrayIconDebug', 1)

OnAutoItExitRegister('OnQuit') ; ### When run in standalone
_OutputWindowCreate() ; ### When run in standalone

_OutputProgressWrite('history.htm creation...')

; Only works with 3.2.0.1 (13th August, 2006) (Release) and newer.
; Use the functions.txt and libfunctions.txt files in docs\autoit\english\txt2htm\syntaxfiles
GenerateChangelog(@ScriptDir & '\html\autoit_changelog.txt', _
		@ScriptDir & '\html\history.htm', _
		@ScriptDir & '\txt2htm\syntaxfiles\functions.txt', _
		@ScriptDir & '\txt2htm\syntaxfiles\libfunctions.txt', _
		@ScriptDir & '\txt2htm\syntaxfiles\macros.txt', '')

_OutputProgressWrite('Complete.' & @CRLF)

Func OnQuit()
	_OutputWaitClosed() ; ### When run in standalone
EndFunc   ;==>OnQuit

Func GenerateChangelog($sChangeLogPath, $sHTMLOutPath, $sNativeList, $sUDFList, $sMacroList, $sLocalRef)
	If Not FileExists($sNativeList) Or Not FileExists($sUDFList) Or Not FileExists($sMacroList) Then
		_OutputProgressWrite('Missing function lists - please re-generate the lists for history.html to be created.' & @CRLF)
		Return False
	EndIf

	; Read the changelog file.
	Local $sData = FileRead($sChangeLogPath)
	If @error Then
		_OutputProgressWrite('There was an error reading the changelog.' & @CRLF)
		Return False
	EndIf

	; Remove old entries (v3.2.0 and older). These aren't supported with this script.
	$sData = StringRegExpReplace($sData, '(?ms)^3\.2\.0\h+.+', '')

	; Convert \r & \n to \r\n.
	_ConvertEOLToCRLF($sData)

	; Convert symbols to entities.
	_ConvertSymbolsToHTMLEntity($sData)

	; Strip <space> and dash at the start of the line.
	$sData = StringRegExpReplace($sData, '(?m:^\h*\-\h*)', '')

	; Remove double blank lines and trailing whitespace.
	_StripEmptyLinesDouble($sData)
	_StripWhitespace($sData)

	; Convert native functions to a URL. Case-sensitive so it matches correctly. Matches Function().
	Local $aAu3API = StringRegExp(FileRead($sNativeList), '(?m)^(\w+)', $STR_REGEXPARRAYGLOBALMATCH)
	For $i = 0 To UBound($aAu3API) - 1
		$sData = StringRegExpReplace($sData, '\b' & $aAu3API[$i] & '\b(\(\))?', '<a href="functions/' & $aAu3API[$i] & '.htm">' & $aAu3API[$i] & '\1</a>')
	Next

	; Convert UDF functions to a URL. Case-sensitive so it matches correctly. Matches _Function().
	$aAu3API = StringRegExp(FileRead($sUDFList), '(?m)^(\w+)', $STR_REGEXPARRAYGLOBALMATCH)
	For $i = 0 To UBound($aAu3API) - 1
		$sData = StringRegExpReplace($sData, '\b' & $aAu3API[$i] & '\b(\(\))?', '<a href="' & $sLocalRef & 'libfunctions/' & $aAu3API[$i] & '.htm">' & $aAu3API[$i] & '\1</a>')
	Next

	; Convert macros. Case-sensitive so it matches correctly. Matches @Macro.
	$aAu3API = StringRegExp(FileRead($sMacroList), '(?m)^@(\w+)', $STR_REGEXPARRAYGLOBALMATCH)
	For $i = 0 To UBound($aAu3API) - 1
		$sData = StringRegExpReplace($sData, '@\b' & $aAu3API[$i] & '\b', '<a href="macros.htm#@' & $aAu3API[$i] & '">@' & $aAu3API[$i] & '</a>')
	Next

	; Convert Trac numbers to a URL with Trac number.
	$sData = StringRegExpReplace($sData, '#(\d+)', '<a href="http://www.autoitscript.com/trac/autoit/ticket/\1" target="_blank">#\1</a>')

	; Parse changelog to an array using a regular expression.
	Local $aSRE = StringRegExp($sData, '(?ms)((?:\d+\.){3}\d+.+?)(?=^(?:\d+\.){3}\d+|\Z)', $STR_REGEXPARRAYGLOBALMATCH), $aSplit = 0, _
			$bIsFirstRun = True, $bIsSectionStart = True, _
			$iIndex = 2

	; HTML header.
	Local $sHTML = '<!DOCTYPE html>' & @CRLF
	$sHTML &= '<html>' & @CRLF
	$sHTML &= '<head>' & @CRLF
	$sHTML &= @TAB & '<title>History</title>' & @CRLF
	$sHTML &= @TAB & '<meta charset="utf-8">' & @CRLF
	$sHTML &= @TAB & '<link href="css/default.css" rel="stylesheet">' & @CRLF
	$sHTML &= '</head>' & @CRLF
	$sHTML &= '<body>' & @CRLF
	$sHTML &= @TAB & '<h1>History</h1>' & @CRLF
	$sHTML &= @TAB & '<p class="c1"><strong>IMPORTANT:</strong> <a href="script_breaking_changes.htm">See here</a> for recent script-breaking changes.</p>' & @CRLF
	$sHTML &= @TAB & '<p>&nbsp;</p>' & @CRLF
	$sHTML &= @TAB & '<p>Here is the summarized history of the changes to AutoIt v3.</p>' & @CRLF
	$sHTML &= @TAB & '<p class="c2">(For the complete technical history click <a href="autoit_changelog.txt">here</a>. For the complete history including <strong>all changes between beta versions</strong> click <a href="autoit_changelog_complete.txt">here</a>.)</p>' & @CRLF
	$sHTML &= @TAB & '<p>&nbsp;</p>' & @CRLF

	Local $bStart = True
	For $i = 0 To UBound($aSRE) - 1 ; Commented out UBound for current release only.
		_StripEmptyLinesStartOfFile($aSRE[$i])
		_StripEmptyLinesEndOfFile($aSRE[$i])

		$aSplit = StringSplit($aSRE[$i], @CRLF, $STR_ENTIRESPLIT)
		If Not @error Then
			$iIndex = 2
			If Not $bStart Then $sHTML &= @TAB & '<br />' & @CRLF
			$bStart = False
			$sHTML &= @TAB & '<p class="c4"><strong class="c3">' & $aSplit[1] & '</strong></p>' & @CRLF ; Release and date.
			If StringStripWS($aSplit[2], $STR_STRIPALL) == '' Then
				$sHTML &= @TAB & '<ul>' & @CRLF
				$iIndex = 3 ; Workaround: If the next entry is blank, then increase by one.
			EndIf
			For $j = $iIndex To $aSplit[0]
				Switch $aSplit[$j] ; Check if the array entry is a section or other.
					Case 'Au3Check:', 'Au3Info:', 'Au3Record:', 'Aut2Exe:', 'AutoIt:', 'AutoIt3Help:', 'AutoItX:', _
							'Big Changes:', 'Exe2Aut:', 'Others:', 'Other Changes:', 'SciTE "lite":', 'UDFs:', _
							'WARNING:'
						If $bIsFirstRun Then ; If this is the first time manipulating the following release, then do the following.
							$bIsFirstRun = False
						Else
							$sHTML &= @TAB & '</ul>' & @CRLF
						EndIf
						$bIsSectionStart = True ; Set to True as we are in a section.
						$sHTML &= @TAB & '<p><strong>' & $aSplit[$j] & '</strong></p>' & @CRLF
						$sHTML &= @TAB & '<ul>' & @CRLF
					Case Else
						If StringStripWS($aSplit[$j], $STR_STRIPALL) = '' Then
							If $bIsSectionStart Then ; If blank line and section then do the following.
								$bIsSectionStart = False

								; If $j is less than the total number of array items and then next line a new section. Add a close ul tag.
								If $j < $aSplit[0] And Not StringRegExp($aSplit[$j + 1], '(?m)^\V+:') Then
									$sHTML &= @TAB & '</ul>' & @CRLF
									If Not ($j = $aSplit[0]) Then ; If not the last entry then add a new ul tag.
										$sHTML &= @TAB & '<ul>' & @CRLF
									EndIf
								Else
									$sHTML &= @TAB & '</ul>' & @CRLF ; Create a blank space.
									$sHTML &= @TAB & '<ul>' & @CRLF
								EndIf
							Else
								$sHTML &= @TAB & '</ul>' & @CRLF ; Create a blank space.
								$sHTML &= @TAB & '<ul>' & @CRLF
							EndIf

						Else ; Else it's either an entry.
							If Not $bIsSectionStart And $j = 2 Then
								$sHTML &= @TAB & '<ul>' & @CRLF
							EndIf
							$sHTML &= @TAB & @TAB & '<li>' & $aSplit[$j] & '</li>' & @CRLF ; Add new entry.
							If $j = $aSplit[0] Then ; If the last entry then add the close ul tag.
								$sHTML &= @TAB & '</ul>' & @CRLF
							EndIf
						EndIf
				EndSwitch
			Next
			If $bIsSectionStart Then ; If section empty then add the close ul tag.
				$sHTML &= @TAB & '</ul>' & @CRLF
			EndIf
		EndIf

		; Reset the variables.
		$bIsFirstRun = True
		$bIsSectionStart = False
	Next
	$sHTML &= '</body>' & @CRLF
	$sHTML &= '</html>' & @CRLF

	; Quick workaround for stray <ul> tags.
	$sHTML = StringRegExpReplace($sHTML, '\h*<ul>\R(\h*<p><strong>)', '\1')

	; Highlight this is a script breaking change statement.
	$sHTML = StringReplace($sHTML, 'THIS IS A SCRIPT BREAKING CHANGE', '<strong>THIS IS A SCRIPT BREAKING CHANGE</strong>', 0, $STR_CASESENSE)
	If Not _FileWrite($sHTMLOutPath, $sHTML, $FO_UTF8) Then
		_OutputProgressWrite('There was an error creating the history.htm file.' & @CRLF)
		Return False
	EndIf

	Return True
EndFunc   ;==>GenerateChangelog
